@@ -5,14 +5,16 @@
 # FIXME: Add IP-XACT support
 import logging
 import os
-from pyparsing import Forward, OneOrMore, Optional, Suppress, Word, alphanums
 import shutil
+import warnings
+
 import yaml
+from edalize import get_edatools
 
 from fusesoc import utils
+from fusesoc.capi2.exprs import Exprs
 from fusesoc.provider import get_provider
 from fusesoc.vlnv import Vlnv
-from edalize import get_edatools
 
 logger = logging.getLogger(__name__)
 
@@ -26,14 +28,14 @@ class File:
         self.logical_name = ""
         if type(tree) is dict:
             for k, v in tree.items():
-                self.name = String(os.path.expandvars(k))
+                self.name = StringWithUseFlags(os.path.expandvars(k))
                 self.file_type = v.get("file_type", "")
                 self.is_include_file = v.get("is_include_file", False)
                 self.include_path = v.get("include_path")
                 self.copyto = v.get("copyto", "")
                 self.logical_name = v.get("logical_name", "")
         else:
-            self.name = String(os.path.expandvars(tree))
+            self.name = StringWithUseFlags(os.path.expandvars(tree))
             self.is_include_file = False  # "FIXME"
 
 
@@ -41,48 +43,46 @@ class Genparams(dict):
     pass
 
 
-class String(str):
+class StringWithUseFlags(str):
+    """ A parsed string with support for use flags. """
+
+    def __init__(self, string):
+        self.exprs = None
+
     def parse(self, flags):
-        _flags = []
-        for k, v in flags.items():
-            if v == True:
-                _flags.append(k)
-            elif v in [False, None]:
-                pass
-            else:
-                _flags.append(k + "_" + v)
+        if self.exprs is None:
+            self.exprs = Exprs(str(self))
+        return self.exprs.expand(flags)
 
-        def cb_conditional(s, l, t):
-            if (t.cond in _flags) != (t.negate == "!"):
-                return t.expr
-            else:
-                return []
 
-        word = Word(alphanums + ":<>.[]_-,=~/^~")
-        conditional = Forward()
-        conditional << (
-            Optional("!")("negate")
-            + word("cond")
-            + Suppress("?")
-            + Suppress("(")
-            + OneOrMore(conditional ^ word)("expr")
-            + Suppress(")")
-        ).setParseAction(cb_conditional)
-        string = word
-        string_list = OneOrMore(conditional ^ string)
-        s = " ".join(string_list.parseString(self.__str__()))
-        logger.debug(
-            "Parsing '{}' with flags {} => {}".format(self.__str__(), str(_flags), s)
+class String(str):
+    """ A plain (unparsed) string. """
+
+    def parse(self, flags):
+        raise RuntimeError(
+            "BUG: Attempted to parse a plain string. This is a bug in FuseSoC. "
+            "Please file an issue at "
+            "https://github.com/olofk/fusesoc/issues/new."
         )
-        return s
 
 
-class StringOrList:
+class StringWithUseFlagsOrList:
     def __new__(cls, *args, **kwargs):
         if type(args[0]) == list:
-            return [String(s) for s in args[0]]
+            return [StringWithUseFlags(s) for s in args[0]]
         elif type(args[0]) == str:
-            return [String(args[0])]
+            return [StringWithUseFlags(args[0])]
+
+
+class StringWithUseFlagsOrDict:
+    def __init__(self, tree):
+        self.params = {}
+        if type(tree) is dict:
+            for k, v in tree.items():
+                self.params = v
+                self.name = StringWithUseFlags(k)
+        else:
+            self.name = StringWithUseFlags(tree)
 
 
 class Section:
@@ -220,10 +220,18 @@ class Core:
 
         dirs = list(set(map(os.path.dirname, src_files)))
         for d in dirs:
-            if not os.path.exists(os.path.join(dst_dir, d)):
-                os.makedirs(os.path.join(dst_dir, d))
+            os.makedirs(os.path.join(dst_dir, d), exist_ok=True)
 
         for f in src_files:
+            if f.startswith(".."):
+                warnings.warn(
+                    "The file {} in {} is not within the directory containing "
+                    "the core file. This is deprecated and will be an error in "
+                    "a future FuseSoC version. A typical solution is to move "
+                    "core file into the root directory of the IP block it "
+                    "describes.".format(f, self.core_file),
+                    FutureWarning,
+                )
             if not os.path.isabs(f):
                 if os.path.exists(os.path.join(self.core_root, f)):
                     shutil.copyfile(
@@ -472,22 +480,30 @@ class Core:
         if not target:
             return ttptttg
 
-        _ttptttg = self._parse_list(flags, target.generate)
+        _ttptttg = []
+        for f in target.generate:
+            pf = f.name.parse(flags)
+            # f.name might end up empty after parse. In that case, we ignore it
+            if pf:
+                _ttptttg.append({"name": pf, "params": f.params})
+
         if _ttptttg:
             self._debug(" Matched generator instances {}".format(_ttptttg))
         for gen in _ttptttg:
-            if not gen in self.generate:
+            gen_name = gen["name"]
+            if not gen_name in self.generate:
                 raise SyntaxError(
                     "Generator instance '{}', requested by target '{}', was not found".format(
-                        gen, target.name
+                        gen_name, target.name
                     )
                 )
-            params = self.generate[gen].parameters or {}
+            gen_inst = self.generate[gen_name]
+            params = utils.merge_dict(gen_inst.parameters or {}, gen["params"])
             t = {
-                "name": gen,
-                "generator": str(self.generate[gen].generator),
+                "name": gen_name,
+                "generator": str(gen_inst.generator),
                 "config": dict(params),
-                "pos": str(self.generate[gen].position or "append"),
+                "pos": str(self.generate[gen_name].position or "append"),
             }
             ttptttg.append(t)
         return ttptttg
@@ -703,7 +719,7 @@ Fileset:
       type : File
       desc : Files in fileset
     - name : depend
-      type : String
+      type : StringWithUseFlags
       desc : Dependencies of fileset
 
 Generate:
@@ -733,7 +749,7 @@ Generators:
       desc : Short description of the generator, as shown with ``fusesoc gen list``
     - name : usage
       type : String
-      desc : A longer description of how to use the generator, including which parameters it uses (as shown with ``fusesoc gen show $generator``
+      desc : A longer description of how to use the generator, including which parameters it uses (as shown with ``fusesoc gen show $generator``).
 
 Target:
   description : A target is the entry point to a core. It describes a single use-case and what resources that are needed from the core such as file sets, generators, parameters and specific tool options. A core can have multiple targets, e.g. for simulation, synthesis or when used as a dependency for another core. When a core is used, only a single target is active. The *default* target is a special target that is always used when the core is being used as a dependency for another core or when no ``--target=`` flag is set.
@@ -751,20 +767,20 @@ Target:
       type : Tools
       desc : Tool-specific options for target
     - name : toplevel
-      type : StringOrList
+      type : StringWithUseFlagsOrList
       desc : Top-level module. Normally a single module/entity but can be a list of several items
   lists:
     - name : filesets
-      type : String
+      type : StringWithUseFlags
       desc : File sets to use in target
     - name : generate
-      type : String
-      desc : Parameterized generators to run for this target
+      type : StringWithUseFlagsOrDict
+      desc : Parameterized generators to run for this target with optional parametrization
     - name : parameters
-      type : String
+      type : StringWithUseFlags
       desc : Parameters to use in target. The parameter default value can be set here with ``param=value``
     - name : vpi
-      type : String
+      type : StringWithUseFlags
       desc : VPI modules to build and include for target
 
 Tools:
@@ -774,16 +790,16 @@ Hooks:
   description : Hooks are scripts that are run at different points in the build process. They are always launched from the work root
   lists:
     - name : pre_build
-      type : String
+      type : StringWithUseFlags
       desc : Scripts executed before the *build* phase
     - name : post_build
-      type : String
+      type : StringWithUseFlags
       desc : Scripts executed after the *build* phase
     - name : pre_run
-      type : String
+      type : StringWithUseFlags
       desc : Scrips executed before the *run* phase
     - name : post_run
-      type : String
+      type : StringWithUseFlags
       desc : Scripts executed after the *run* phase
 
 Parameter:
@@ -799,8 +815,8 @@ Parameter:
       type : String
       desc : Description of the parameter, as can be seen with ``fusesoc run --target=$target $core --help``
     - name : paramtype
-      type : String
-      desc : Specifies type of parameter. Legal values are *cmdlinearg* for command-line arguments directly added when running the core, *generic* for VHDL generics, *plusarg* for verilog plusargs, *vlogdefine* for verilog `define or *vlogparam* for verilog top-level parameters. All paramtypes are not valid for every backend. Consult the backend documentation for details.
+      type : StringWithUseFlags
+      desc : Specifies type of parameter. Legal values are *cmdlinearg* for command-line arguments directly added when running the core, *generic* for VHDL generics, *plusarg* for verilog plusargs, *vlogdefine* for Verilog `` `define`` or *vlogparam* for verilog top-level parameters. All paramtypes are not valid for every backend. Consult the backend documentation for details.
     - name : scope
       type : String
       desc : "**Not used** : Kept for backwards compatibility"
@@ -921,8 +937,11 @@ _generate_classes(capi2_data, Section)
 
 def gen_doc():
     c = capi2_data.copy()
-    s = """CAPI2
-=====
+    s = """.. _ref_capi2:
+
+***************
+CAPI2 Reference
+***************
 
 CAPI2 (Core API version 2) describes the properties of a core as a YAML data structure.
 
@@ -952,23 +971,50 @@ Specifies how to fetch the core. The presence of a provider section indicates th
 
 String
 ~~~~~~
-String is a string that can contain CAPI2 expressions that are evaulated during parsing.
 
-CAPI2 expressions are used to evaluate an exprssion only if a flag is set or unset. The general form is *flag_is_set ? ( expression )* to evaluate *expression* if flag is set or *!flag_is_set ? ( expression )* to evaluate *expression* if flag is not set.
+String is an unparsed (plain) string.
+
+StringWithUseFlags
+~~~~~~~~~~~~~~~~~~
+
+StringWithUseFlags is a string that can contain CAPI2 expressions that are evaluated during parsing.
+
+CAPI2 expressions are used to evaluate an expression only if a flag is set or unset.
+The general form is *flag_is_set ? ( expression )* to evaluate *expression* if flag is set or *!flag_is_set ? ( expression )* to evaluate *expression* if flag is not set.
 
 **Example** Only include fileset *verilator_tb* when the target is used with verilator
 
 ``filesets : [rtl, tb, tool_verilator? (verilator_tb)]``
 
-StringOrList
-~~~~~~~~~~~~
+StringWithUseFlagsOrDict
+~~~~~~~~~~~~~~~~~~~~~~~~
 
-Item is allowed to be either a `String`_ or a list of `String`_
+Item is allowed to be either a `StringWithUseFlags`_ or a dict of `StringWithUseFlags`_.
+
+StringWithUseFlagsOrList
+~~~~~~~~~~~~~~~~~~~~~~~~
+
+Item is allowed to be either a `StringWithUseFlags`_ or a list of `StringWithUseFlags`_
 
 Vlnv
 ~~~~~~
 :-separated VLNV (Vendor, Library, Name, Version) identifier
 
+Dict
+~~~~
+
+A dictionary.
+
+Bool
+~~~~
+
+A boolean.
+
+Integer
+~~~~~~~
+
+An integer.
+
 List
 ~~~~
 
