@@ -2,20 +2,11 @@
 # Licensed under the 2-Clause BSD License, see LICENSE for details.
 # SPDX-License-Identifier: BSD-2-Clause
 
-from collections import OrderedDict
+import configparser
 import logging
-
-import sys
-
-if sys.version[0] == "2":
-    import ConfigParser as configparser
-    from ConfigParser import SafeConfigParser as CP
-else:
-    import configparser
-    from configparser import ConfigParser as CP
-
 import os
-import importlib
+import sys
+from configparser import ConfigParser as CP
 
 from fusesoc.librarymanager import Library
 
